import os
from pymongo import MongoClient

MONGO_HOST = os.environ.get('SENSE_MONGO_HOST') or 'localhost'
MONGO_PORT = int(os.environ.get('SENSE_MONGO_PORT')) or 27017
MONGO_DB = os.environ.get('SENSE_MONGO_DB')
MONGO_USERNAME = os.environ.get('SENSE_MONGO_USERNAME')
MONGO_PASSWORD = os.environ.get('SENSE_MONGO_PASSWORD')
MONGO_AUTHSOURCE = os.environ.get('SENSE_MONGO_AUTHSOURCE')
COLLECTION = os.environ.get('SENSE_COLLECTION')

mongo = MongoClient(
    host=MONGO_HOST,
    port=MONGO_PORT,
    username=MONGO_USERNAME,
    password=MONGO_PASSWORD,
    authSource=MONGO_AUTHSOURCE,
)
db = mongo.get_database(MONGO_DB)
col = db.get_collection(COLLECTION)



import os

import click

from sense_requests import constants
from sense_requests.core.client import client
from sense_requests.core.config import config
from sense_requests.config import VERSION


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(f'v{VERSION}')
    ctx.exit()


@click.group()
@click.option('--version', '-v', is_flag=True, callback=print_version, expose_value=False, is_eager=True,
              help='show version of SDK')
def cli():
    pass


@click.command('login', help='login to the platform and get token')
@click.option('--username', '-u')
@click.password_option('--password', '-p', confirmation_prompt=False)
@click.option('--api_address', '-a')
def login(username=None, password=None, api_address=None):
    if username is not None:
        config.data.username = username
    if password is not None:
        config.data.password = password
    if api_address is not None:
        config.data.api_address = api_address
    config.save()
    client.update_token()


@click.command('config', help='set config info')
@click.option('--username', '-u')
@click.password_option('--password', '-p', confirmation_prompt=False)
@click.option('--api_address', '-a')
def config_(username=None, password=None, api_address=None):
    if username is not None:
        config.data.username = username
    if password is not None:
        config.data.password = password
    if api_address is not None:
        config.data.api_address = api_address
    config.save()
    print('config has been saved')


@click.command('check', help='check the connection and update token')
def check():
    client.update_token()


@click.command('nodes', help='list the nodes')
def nodes():
    client.list_nodes()


@click.command('spiders', help='list the spiders')
def spiders():
    client.list_spiders()


@click.command('schedules', help='list the schedules')
def schedules():
    client.list_schedules()


@click.command('tasks', help='list the tasks')
@click.option('--number', '-n', help='number of tasks')
def tasks(number=None):
    client.list_tasks(number)


@click.command('upload', help='upload a spider')
@click.option('--type', '-t', type=click.Choice(['customized', 'configurable']), default=constants.Spider.CUSTOMIZED,
              help='spider type')
@click.option('--directory', '-d', help='directory path, for customized spiders')
@click.option('--name', '-n', help='spider name')
@click.option('--col', '-c', help='spider results collection')
@click.option('--display_name', '-N', help='spider display name')
@click.option('--command', '-m', help='spider execution command')
@click.option('--id', '-i', help='spider id')
@click.option('--spiderfile', '-f', help='Spiderfile path')
def upload(type=None, directory=None, name=None, col=None, display_name=None, command=None, id=None, spiderfile=None):
    # TODO: finish all functionality
    if type is None:
        type = constants.Spider.CUSTOMIZED

    if type == constants.Spider.CUSTOMIZED:
        # customized spider
        if directory is None:
            directory = os.path.abspath(os.curdir)
            client.upload_customized_spider(directory, name, col, display_name, command, id)
    elif type == constants.Spider.CONFIGURABLE:
        # configurable spider
        pass


@click.command('settings', help='get settings of a scrapy project')
@click.option('--directory', '-d', help='directory path, for scrapy spiders')
def settings(directory=None):
    client.settings(directory)


@click.command('items', help='get items of a scrapy project')
@click.option('--directory', '-d', help='directory path, for scrapy spiders')
def items(directory=None):
    client.items(directory)


@click.command('pipelines', help='get pipelines of a scrapy project')
@click.option('--directory', '-d', help='directory path, for scrapy spiders')
@click.option('--name', '-n', help='pipeline name')
@click.option('--delete', '-D', help='whether to delete the pipeline')
def pipelines(directory=None, name=None, delete=None):
    client.pipelines(directory, name, delete)


@click.command('find_spider_filepath', help='get filepath from spider name of a scrapy project')
@click.option('--directory', '-d', help='directory path, for scrapy spiders')
@click.option('--spider_name', '-n', help='spider name')
def find_spider_filepath(directory=None, spider_name=None):
    if spider_name is None:
        print('spider_name is required')
        return
    client.find_spider_filepath(directory, spider_name)


def main():
    cli.add_command(check)
    cli.add_command(config_)
    cli.add_command(find_spider_filepath)
    cli.add_command(items)
    cli.add_command(login)
    cli.add_command(nodes)
    cli.add_command(pipelines)
    cli.add_command(settings)
    cli.add_command(schedules)
    cli.add_command(tasks)
    cli.add_command(spiders)
    cli.add_command(upload)

    cli()


if __name__ == '__main__':
    main()



import configparser
import importlib
import json
import os
import sys
from zipfile import ZipFile, ZIP_DEFLATED

from prettytable import PrettyTable

from sense_requests.core import SENSE_TMP
from sense_requests.core.config import config
from sense_requests.core.request import Request
from sense_requests.utils.scrapy import get_scrapy_cfg, get_items_fields, get_pipelines, get_spider_filepath


def zip_dir(start_dir, target_path):
    start_dir = start_dir  # 要压缩的文件夹路径
    file_new = target_path  # 压缩后文件夹的名字

    z = ZipFile(file_new, 'w', ZIP_DEFLATED)
    for dir_path, dir_names, file_names in os.walk(start_dir):
        f_path = dir_path.replace(start_dir, '')  # 这一句很重要，不replace的话，就从根目录开始复制
        f_path = f_path and f_path + os.sep or ''  # 实现当前文件夹以及包含的所有文件的压缩
        for filename in file_names:
            print(f_path + filename)
            z.write(os.path.join(dir_path, filename), f_path + filename)
    z.close()
    return file_new


def get_zip_file(input_path, result):
    files = os.listdir(input_path)
    for file in files:
        filepath = os.path.join(input_path, file)
        if os.path.isdir(filepath):
            get_zip_file(filepath, result)
        else:
            result.append(filepath)


def zip_file_path(input_path, target_path):
    f = ZipFile(target_path, 'w', ZIP_DEFLATED)
    file_list = []
    get_zip_file(input_path, file_list)
    for file in file_list:
        f.write(file)
    f.close()
    return target_path


class Client(object):
    def __init__(self):
        pass

    @staticmethod
    def list(columns, items):
        tb = PrettyTable()
        tb.field_names = columns
        for item in items:
            row = []
            for col in columns:
                row.append(item.get(col))
            tb.add_row(row)
        print(tb)
        print(f'total: {len(items)}')

    @staticmethod
    def update_token():
        data = Request.post('/login', data={
            'username': config.data.username,
            'password': config.data.password,
        })
        if data.get('error'):
            print('error: ' + data.get('error'))
            print('error when logging-in')
            return
        config.data.token = data.get('data')
        config.save()
        print('logged-in successfully')

    @staticmethod
    def list_nodes():
        data = Request.get('/nodes')
        if data.get('error'):
            print('error: ' + data.get('error'))
        items = data.get('data') or []
        columns = ['_id', 'name', 'status', 'create_ts', 'update_ts']
        Client.list(columns, items)

    @staticmethod
    def list_spiders():
        data = Request.get('/spiders', {'page_num': 1, 'page_size': 99999999})
        if data.get('error'):
            print('error: ' + data.get('error'))
        items = (data.get('data').get('list') or []) if data.get('data') is not None else []
        columns = ['_id', 'name', 'display_name', 'type', 'col', 'create_ts', 'update_ts']
        Client.list(columns, items)

    @staticmethod
    def list_schedules():
        data = Request.get('/schedules', {'page_num': 1, 'page_size': 99999999})
        if data.get('error'):
            print('error: ' + data.get('error'))
        items = data.get('data') or []
        columns = ['_id', 'name', 'spider_name', 'run_type', 'cron', 'create_ts', 'update_ts']
        Client.list(columns, items)

    @staticmethod
    def list_tasks(number=10):
        data = Request.get('/tasks', {'page_num': 1, 'page_size': number})
        if data.get('error'):
            print('error: ' + data.get('error'))
        items = data.get('data') or []
        columns = ['_id', 'status', 'node_name', 'spider_name', 'error', 'result_count', 'create_ts', 'update_ts']
        Client.list(columns, items)

    @staticmethod
    def upload_customized_spider(directory=None, name=None, col=None, display_name=None, command=None, id=None):
        # check if directory exists
        if not os.path.exists(directory):
            print(f'error: {directory} does not exist')
            return

        # compress the directory to the zipfile
        target_path = os.path.join(SENSE_TMP, name or os.path.basename(directory)) + '.zip'
        zip_dir(directory, target_path)

        # upload zip file to server
        if id is None:
            res = Request.upload('/spiders', target_path, data={
                'name': name,
                'display_name': display_name,
                'col': col,
                'command': command,
            })
            if res.get('error'):
                print('error: ' + res.get('error'))
                return
        else:
            res = Request.upload(f'/spiders/{id}/upload', target_path)
            if res.get('error'):
                print('error: ' + res.get('error'))
                return
        print('uploaded successfully')

    @staticmethod
    def upload_configurable_spider(directory=None, name=None, col=None, display_name=None, command=None, id=None):
        pass

    @staticmethod
    def settings(directory=None):
        if directory is None:
            directory = os.path.abspath(os.curdir)
        os.chdir(directory)

        cp = get_scrapy_cfg()

        settings_mod_name = cp.get('settings', 'default')

        sys.path.insert(0, directory)
        settings = importlib.import_module(settings_mod_name)

        data = []
        for key in [key for key in dir(settings) if not key.startswith('__')]:
            data.append({
                'key': key,
                'value': getattr(settings, key),
            })

        print(json.dumps(data))

    @staticmethod
    def items(directory=None):
        if directory is None:
            directory = os.path.abspath(os.curdir)
        os.chdir(directory)

        cp = get_scrapy_cfg()

        settings_mod_name = cp.get('settings', 'default')
        project_name = settings_mod_name.split('.')[0]
        items_mod_name = f'{project_name}.items'

        sys.path.insert(0, directory)
        items = importlib.import_module(items_mod_name)

        # get list of all items fields
        data = get_items_fields(items)
        print(json.dumps(data))

    @staticmethod
    def pipelines(directory=None, name=None, delete=None):
        if directory is None:
            directory = os.path.abspath(os.curdir)
        os.chdir(directory)

        cp = get_scrapy_cfg()

        settings_mod_name = cp.get('settings', 'default')
        project_name = settings_mod_name.split('.')[0]
        pipelines_mod_name = f'{project_name}.pipelines'

        sys.path.insert(0, directory)
        pipelines = importlib.import_module(pipelines_mod_name)

        # get list of all pipelines
        if name is None:
            data = get_pipelines(pipelines)
            print(json.dumps(data))

    @staticmethod
    def find_spider_filepath(directory=None, name=None):
        if directory is None:
            directory = os.path.abspath(os.curdir)
        os.chdir(directory)
        sys.path.insert(0, directory)

        cp = get_scrapy_cfg()

        settings_mod_name = cp.get('settings', 'default')
        project_name = settings_mod_name.split('.')[0]

        spiders_path = os.path.join(directory, project_name, 'spiders')
        filenames = os.listdir(spiders_path)

        filepath = get_spider_filepath(filenames, project_name, name)
        print(filepath)


client = Client()



import os
from pymongo import ASCENDING
import datetime


def get_task_id():
    return os.environ.get('SENSE_TASK_ID')


def save_item(item):
    from sense_requests.db import col
    col.create_index([("_ttl_time", ASCENDING)], expireAfterSeconds=3600 * 24)
    item['ttl_time'] = datetime.datetime.utcnow()
    item['task_id'] = get_task_id()
    col.save(item)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

#                                                           
# Copyright (C)2017 SenseDeal AI, Inc. All Rights Reserved  
#                                                           

"""                                                   
File: setup.py
Author: lzl
E-mail: zll@sensedeal.ai
Last modified: 2020/3/9
Description:                                              
"""

from setuptools import setup, find_packages
import sense_requests.config

setup(
    name='sense-requests-sdk',
    version=sense_requests.config.VERSION,
    packages=find_packages(),
    url='',
    license='BSD License',
    author='kafka0102',
    author_email='yujianjia@sensedeal.ai',
    description='Python SDK for Sense-Requests',
    long_description='',
    long_description_content_type="text/markdown",
    scripts=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'Click',
        'requests',
        'prettytable',
        'scrapy',
        'pymongo',
    ],
    entry_points={
        'console_scripts': [
            'sense_requests=requests:main'
        ]
    }
)



from sense_requests.db import col
from sense_requests.utils import get_task_id
from pymongo import ASCENDING
import datetime


class SenseMongoPipeline(object):
    def process_item(self, item, spider):
        item_dict = dict(item)
        item_dict['task_id'] = get_task_id()
        col.create_index([("_ttl_time", ASCENDING)], expireAfterSeconds=3600 * 24)
        item_dict['ttl_time'] = datetime.datetime.utcnow()
        col.save(item_dict)
        return item



import os

SENSE_ROOT = os.path.join(os.environ.get('HOME'), '.sense_requests')
SENSE_TMP = os.path.join(SENSE_ROOT, 'tmp')

if not os.path.exists(SENSE_ROOT):
    os.mkdir(SENSE_ROOT)

if not os.path.exists(SENSE_TMP):
    os.mkdir(SENSE_TMP)
